#!/usr/bin/env python
# -*- coding: utf-8 -*-

import piou

from piou.config import phase
import logging
import os

logger = logging.getLogger("inpcheck")

class InputValidator(piou.PHASE_IO_utility):

    dirs_checked = []
    specfile_specified=False

    def fill_opts(self,parser):
        parser.add_option("-s","--specfile",dest="specfile",\
        help="specify the name of the specification file.",default=None)
        parser.add_option("-r","--recursive",action="store_true",dest="recursive",\
        help="set this option in order to apply inpcheck recusively.",default=False)

    def initialize(self,opts):
        self.recursive=opts.recursive

    def __init__(self):
        self.program = 'phase'

    def do_inpcheck(self):
        logger.info("-- running the input validator --")
        logger.info("specfile : "+self.program+".spec")
        self._inpcheck(".",self.recursive)
        if len(self.dirs_checked)>=2:
            logger.debug("checked a total of : "+str(len(self.dirs_checked))+" directories")
            for dir in self.dirs_checked:
                logger.debug(" "+dir)

    def _inpcheck(self, dir, recur):
        os.chdir(dir)
        logger.info("checking directory : "+os.getcwd())
        files=os.listdir(".")
        sfile=None
        if not self.specfile_specified:
            sfile=self.program+'.spec'
            specfile_specified=True
        #if 'file_names.data' in files:
        self.dirs_checked.append(os.getcwd())
        inp = phase.Input(sfile,coord_file=None)
        if inp.inpfile_exists():
            try:
                inp.validate()
            except phase.InputValidationError:
                logger.error("input validation FAILED!")
                #pass
                #logger.error("input validation FAILED!")
            nwarn = inp.get_nwarn()
            nerror = inp.get_nerror()
            if nwarn == 0 and nerror == 0:
                logger.info("no errors/warnings were found in "+os.getcwd())
            elif nerror==0:
                logger.warn("found "+str(nwarn)+" warnings in "+os.getcwd())
                logger.warn("check the log for details.")
            else:
                logger.error("found "+str(nerror)+" errors and "+str(nwarn)+" warnings in "+os.getcwd())
                logger.error("check the log for details.")
        elif not recur:
            logger.error(" input file ["+inp.get_file_name()+"] does not exist under "+os.getcwd())

        if recur:
            for file in files:
                if os.path.isdir(file):
                    self._inpcheck(file,True)
            os.chdir("..")
            logger.info("")

    def get_description(self):
        return \
        "input data validator utility for PHASE"

    def get_name(self):
        return 'the input file validator'

    def run(self):
        self.do_inpcheck()

if __name__ == "__main__":
    piou.run_main(InputValidator)




''' module for selecting and instantiating atomic configurations '''

import os
import sys
import logging
logger = logging.getLogger('piou.config.genconfig')

from piou.config import phase,openmx,cif,cube,xyz,vasp,xsf,dmol,obabel,lammps

from piou.util import pyutil

coordinate_file_type=(\
                  'phase_input','phase_output',\
                  'VASP_input','VASP_output',\
                  'OpenMX_input','OpenMX_output',\
                  'XSF','xyz','cube','cif','dmol',\
                  'LAMMPS_input', 'LAMMPS_output', \
                  'OpenBabel_interface'\
                  )

coord_generators={\
                  'phase_input':phase.Input,'phase_output':phase.F_DYNM,\
                  'VASP_input':vasp.VASP_input,'VASP_output':vasp.VASP_output,\
                  'OpenMX_input':openmx.OMX_input,'OpenMX_output':openmx.OMX_output,\
                  'LAMMPS_input':lammps.LAMMPS_input, 'LAMMPS_output':lammps.LAMMPS_output, \
                  'XSF':xsf.XSF,'xyz':xyz.XYZ,\
                  'cube':cube.Cube,'cif':cif.CIF,\
                  'dmol':dmol.DMol,'OpenBabel_interface':obabel.OpenBabelInterface\
                  }

def get_all_supported_generators(mode='w'):
    ''' return a list of all supported AtomicConfiguration generators '''
    tmpgenerators={}
    typetype=[]
    for ty in coordinate_file_type:
        tmpgen = coord_generators[ty](None)
        tmpgenerators[ty]=tmpgen
        if tmpgen.check_support() and not (mode=='w' and not tmpgen.supports_export()) and \
            not (mode=='r' and not tmpgen.supports_import()):
            typetype.append(ty)
    ret = []
    for ft in typetype:
        tmp=tmpgenerators[ft]
        stypes=tmp.get_all_subtypes(mode=mode)
        if stypes is not None:
            for stype in stypes:
                tmpgen = coord_generators[ft](None)
                tmpgen.set_subtype(stype)
                ret.append(tmpgen)
        else:
            ret.append(tmp)
    return ret

def instantiate_coords_generator(coordtyp,coordinate_file):
    coords_generator=None
    if coordtyp in coordinate_file_type:
        coordgen = coord_generators[coordtyp]
        if coordgen is None:
            logger.error('invalid coord-generator : '+coordtyp)
        else:
            coords_generator = coordgen(coord_file=coordinate_file)

    if coords_generator is None or not coords_generator.is_valid():
        logger.error("failed to instantiate the coords generator. ")
        return None

    return coords_generator

def select_coordinate_file_interactively(msg,default_typ,typ_lambda=None,msg_frame=None,must_exist=True,\
    mustnot_exist=False,ask_frame_no=True,ask_mode=False,extended_frame_spec=False,check_export_support=False,
    mode=None):
    ''' select the coordinate file interactively. 
        will return a tuple with the following elements : 
        (coord_generator,coordinate_type,coordinate_file,frameno) '''
    lam = lambda f:os.path.exists(f)
    invalid='file does not exist: '
    if not must_exist:
        lam=lambda f:True
    if mustnot_exist:
        lam=lambda f:not os.path.exists(f)
        invalid='file exists: '
    typetype=[]
    tmpgenerators={}
    for ty in coordinate_file_type:
        tmpgen = coord_generators[ty](None)
        tmpgenerators[ty]=tmpgen
        if not tmpgen.check_support() or\
            (mode == 'w' and not tmpgen.supports_export()) or\
            (mode == 'r' and not tmpgen.supports_import()):
            continue
        typetype.append(ty)
        
    while True:
        coordinate_type = pyutil.interactive(msg='select the type of '+msg,\
        condition=typ_lambda,default=default_typ,choice=typetype,return_index=False,\
        ask_mode=ask_mode)
        if check_export_support and not tmpgenerators[coordinate_type].supports_export():
            logger.error("output support unavailable for coordinate type '"+coordinate_type+"'")
        else:
            break

    subtype = tmpgenerators[coordinate_type].select_subtype(mode=mode) 

    defname = tmpgenerators[coordinate_type].get_defaultname(subtype)
    coordinate_file = pyutil.interactive(msg="the name "+msg, condition=lam,typ=str,\
                      default=defname,msg_invalid_input=invalid, ask_mode=ask_mode)

    coords_generator = instantiate_coords_generator(coordinate_type,coordinate_file)
    coords_generator.set_subtype(subtype)

    frame_no = -1
    iframe   = -1
    start_end_skip=None
    if coords_generator.supports_frame() and ask_frame_no:
        if msg_frame is None:
            msg_frame = "the frame no. (the last frame will be adopted if a negative value is specified)"
        while True:
            frame_no = pyutil.interactive(msg=msg_frame, condition=lambda ii : True,typ=str,default="-1",\
            ask_mode=ask_mode)
            try:
                iframe = int(frame_no)
                break
            except ValueError:
                pass
            start_end_skip=[]
            ff = frame_no.strip().split(',')
            if len(ff)<3:
                print('enter an int')
                continue
            else:
                try:
                    start_end_skip.append(int(ff[0]))
                    start_end_skip.append(int(ff[1]))
                    start_end_skip.append(int(ff[2]))
                    break
                except ValueError:
                    print('enter the start frame, end frame and the number of frames to skip '+\
                    'by integers delimited by commas')
                    continue

    if not extended_frame_spec:
        return (coords_generator,coordinate_type,coordinate_file,iframe)
    else:
        return (coords_generator,coordinate_type,coordinate_file,iframe,start_end_skip)




"""
optparse -- forward-compatibility wrapper for use with Python 2.2.x and
earlier.  If you import from 'optparse' rather than 'optik', your code
will work on base Python 2.3 (and later), or on earlier Pythons with
Optik 1.4.1 or later installed.
"""

from optik import __version__
from optik import *



import logging
logger = logging.getLogger('piou.config.cube')
from piou import config
from piou.data import elements

class Cube(config.AtomConfigGenerator):
    cubefile=None
    def __init__(self,coord_file):
        self.cubefile = coord_file

    def supports_volumetric_data(self):
        return True

    def _gen_atomic_configuration(self):
        f=open(self.cubefile,'r')
        linecount=0
        nat=None
        avec=None
        bvec=None
        cvec=None
        conf=None
        thevdata=[]
        in_vdata=False
        unit = config.BOHR
        ndiv = []
        shift=[0.0,0.0,0.0]
        for line in f:
            line=line.strip()
            words=line.split()
            if in_vdata:
                for word in words:
                    try:
                        thevdata.append(float(word.strip().lower().replace('d','e')))
                    except ValueError:
                        logger.error('invalid volumetric data!!')
                        return None
            else:
                linecount += 1
                if linecount==3:
                    try:
                        nat=int(words[0])
                        if len(words)>=4:
                            for i in range(3):
                                shift[i]=float(words[i+1])
                    except ValueError:
                        logger.error('failed to retrieve the number of atoms')
                        return None
                elif linecount==4:
                    if len(words)>=4:
                        avec=[]
                        unit=self._do_cellvec(words[0],(words[1],words[2],words[3]),avec,ndiv)
                elif linecount==5:
                    if len(words)>=4:
                        bvec=[]
                        unit=self._do_cellvec(words[0],(words[1],words[2],words[3]),bvec,ndiv)
                elif linecount==6:
                    if len(words)>=4:
                        cvec=[]
                        unit=self._do_cellvec(words[0],(words[1],words[2],words[3]),cvec,ndiv)

                if unit is None:
                    return None

                if avec is None or bvec is None or cvec is None:
                    continue

                if conf is None:
                    conf = config.AtomConfig(\
                    coordinate_system=config.CARTESIAN, length_unit=unit,\
                    a_vector=avec,b_vector=bvec,c_vector=cvec)
                    continue

                if len(words)<5:
                    continue

                try:
                    elem=elements.ElementInfo().get_element_name_from_atomic_number(int(words[0]))
                    rx = float(words[2])
                    ry = float(words[3])
                    rz = float(words[4])
                    conf.add_atom(element=elem,rx=rx-shift[0],ry=ry-shift[1],rz=rz-shift[2])
                except ValueError:
                    logger.error('invalid line : '+line)
                    return None
                except KeyError:
                    logger.error('invalid line : '+line)
                    return None
                if conf.get_num_atom()==nat:
                    in_vdata=True
        f.close()
        self._do_volumetric_data(conf,ndiv,thevdata)
        logger.info(str(conf.get_volumetric_data()[0]))
        return [conf]

    def _do_cellvec(self,strndiv,strvec,thevec,thendiv):
        unit=config.BOHR
        try:
            ndiv=float(strndiv)
            if ndiv<0:
                ndiv=-ndiv
                unit=config.ANGSTROM
            thevec.append(ndiv*float(strvec[0]))
            thevec.append(ndiv*float(strvec[1]))
            thevec.append(ndiv*float(strvec[2]))
            thendiv.append(ndiv)
        except ValueError:
            logger.error('failed to retrieve the cell vector')
            return None
        return unit

    def _do_volumetric_data(self,conf,ndiv,thevdata):
        unitcell = conf.get_unit_cell()
        if not unitcell.ready():
            return
        vdata = config.VolumetricData(unitcell=unitcell,n1=ndiv[0],n2=ndiv[1],n3=ndiv[2],vdata=thevdata)
        conf.add_volumetric_data(vdata)

    def export_atomic_configuration(self,atomic_coordinates,to_file=None,frame_no=None,all_frames=False):
        import time
        from piou.data import elements
        atomic_coordinate = atomic_coordinates[0]
        natm=atomic_coordinate.get_num_atom()
        vd=atomic_coordinate.get_volumetric_data()
        if vd is None:
            logger.error('Volumetric Data must be defined in order to export data in Gaussian Cube format')
            return 
        elif not vd[0].has_enough_data():
            logger.warn('the number of data for the volumetric data is not consistent')
        vdata = vd[0]

        file=open(to_file,'w')
        file.write('Gaussian Cube file generated by the PHASE IO utility\n')
        file.write(str('created on : '+time.strftime('%x %X'))+'\n')

        origin = vdata.get_origin()
        strorigin='0.0 0.0 0.0'
        if len(origin)>=3:
            strorigin = str(origin[0])+' '+str(origin[1])+' '+str(origin[2])
        file.write(str(natm)+' '+strorigin+'\n')

        ndiv = vdata.get_ndiv()
        (div0,div1,div2)  = vdata.get_div(to_unit=config.BOHR)
        file.write(str(int(ndiv[0]))+' '+str(div0[0])+' '+str(div0[1])+' '+str(div0[2])+'\n')
        file.write(str(int(ndiv[1]))+' '+str(div1[0])+' '+str(div1[1])+' '+str(div1[2])+'\n')
        file.write(str(int(ndiv[2]))+' '+str(div2[0])+' '+str(div2[1])+' '+str(div2[2])+'\n')

        for atom in atomic_coordinate:
            atomstr=str(elements.ElementInfo().get_element_info(atom.get_element_name())['atomic_number'])+' 1 '
            (rx,ry,rz) = atom.get_pos(mode=config.CARTESIAN, to_unit=config.BOHR)
            atomstr += ' '+str(rx)+' '+str(ry)+' '+str(rz)
            file.write(atomstr+'\n')

        vol = vdata.get_vdata()
        nrow=6
        currrow=0
        for v in vol:
            currrow += 1
            file.write(str(v)+' ')
            if currrow%nrow==0:
                file.write('\n')
                currrow=0
        file.write('\n')

# can also be done in the following way:
#        n0=int(ndiv[0])
#        n1=int(ndiv[1])
#        n2=int(ndiv[2])
#        for i0 in range(n0):
#            for i1 in range(n1):
#                for i2 in range(n2):
#                    v = vdata.get_vdata_at(i0,i1,i2)
#                    currrow += 1
#                    if currrow%nrow==0:
#                        file.write('\n')
#                        currrow=0
#                    file.write(str(v)+' ')

        file.flush()
        file.close()

    def get_name(self):
        return 'Gaussian cube format'

    def get_defaultname(self,args):
        return 'nfchr.cube'




import logging
logger = logging.getLogger('piou.config.lammps')
from piou import config
from piou.config import AtomConfigGenerator

class LAMMPS_input(AtomConfigGenerator):
    coord_file = None
    valid = True
    def __init__(self,coord_file=None):
        if coord_file is None:
            return
        self.coord_file = coord_file

    def supports_frame(self):
        return False

    def _gen_atomic_configuration(self):
        pass

    def supports_import(self):
        return False

    def export_atomic_configuration(self,atomic_coordinates,to_file=None,frame_no=None,all_frames=True):
        import time
        atomic_coordinate = atomic_coordinates[0]
        natm = atomic_coordinate.get_num_atom()
        elist = atomic_coordinate.get_elem_list()
        file = open(to_file,'w')
        file.write('LAMMPS input file generated by the PHASE I/O utility\n');
        file.write(str('# created on : '+time.strftime('%x %X'))+'\n')
        file.write('\n')
        file.write(str(natm)+' atoms\n')
        file.write(str(len(elist))+' atom types\n')
        unit_cell = atomic_coordinate.get_unit_cell()
        if atomic_coordinate.is_cell_ready():
            (a,b,c,alph,bet,gam) = unit_cell.get_lattice_constant() 
            unit_cell.set_lattice_constant(a,b,c,alph,bet,gam)
            (avec,bvec,cvec) = unit_cell.get_lattice_vector(to_unit=config.ANGSTROM)
            file.write('0 '+str(avec[0])+' xlo xhi\n')
            file.write('0 '+str(bvec[1])+' ylo yhi\n')
            file.write('0 '+str(cvec[2])+' zlo zhi\n')
            file.write(str(bvec[0])+' '+str(cvec[0])+' '+str(cvec[1])+' xy xz yz\n')
        file.write('\n')
        file.write('Atoms\n')
        file.write("\n")
        for i in range(natm):
            atomID=i+1
            atom = atomic_coordinate.get_atom_at(i)
            cpos = atom.get_pos(mode=config.CARTESIAN, to_unit=config.ANGSTROM)
            elem = atom.get_element_name()
            eid = -1
            for j in range(len(elist)):
                if elem==elist[j]:
                    eid = j+1
                    break
            file.write(str(atomID)+' '+str(eid)+' '+str(cpos[0])+' '+str(cpos[1])+' '+str(cpos[2])+'\n')
        file.flush()
        file.close()

    def get_name(self):
        return 'LAMMPS input format'

    def get_defaultname(self,args):
        return "coord.lammps"

class LAMMPS_output(AtomConfigGenerator):
    coord_file = None
    valid = True
    def __init__(self,coord_file=None):
        if coord_file is None:
            return
        self.coord_file = coord_file

    def supports_frame(self):
        return True

    def _gen_atomic_configuration(self):
        #dumpfile = file(self.coord_file)
        dumpfile = open(self.coord_file)
        logger.info("parsing LAMMPS dump file : "+self.coord_file)
        natm=0
        currnatm = -1
        in_natm = False
        in_cell = False
        in_coords = False
        cell_count=0
        a0=None
        b0=None
        b1=None
        c0=None
        c1=None
        c2=None
        xlo=0.0
        xhi=0.0
        ylo=0.0
        yhi=0.0
        zlo=0.0
        zhi=0.0
        xy=0.0
        xz=0.0
        yz=0.0
        ret = []
        atom_config=None
        unit_cell=None
        element_index=1
        for line in dumpfile:
            line = line.strip()
            words = line.split()
            if line.startswith('ITEM'):
                if len(words)<2: 
                    continue
                keyw = words[1]
                if keyw=='NUMBER': 
                    in_natm = True
                    continue
                elif keyw=='BOX':
                    in_cell = True
                elif keyw=='ATOMS':
                    if atom_config is None:
                        logger.error('the unit cell is seemingly undefined...')
                        break
                    if 'element' in line:
                        for i in range(len(words)):
                            if words[i]=='element':
                                element_index = i-2
                    in_coords=True
                continue
            if in_natm:
                try:
                    natm = int(line)
                    in_natm = False
                    continue
                except ValueError:
                    logger.error("invalid number of atoms : "+line)
                    in_natm = False
                    break
            if in_cell:
                if len(words)<2:
                    continue
                try:
                    if cell_count==0: 
                        xlo=float(words[0])
                        xhi=float(words[1])
                        xy = 0
                        if len(words)>2:
                            xy=float(words[2])
                        cell_count += 1
                        continue
                    elif cell_count==1:
                        ylo=float(words[0])
                        yhi=float(words[1])
                        xz = 0
                        if len(words)>2:
                            xz=float(words[2])
                        cell_count += 1
                        continue
                    elif cell_count==2:
                        zlo=float(words[0])
                        zhi=float(words[1])
                        yz = 0
                        if len(words)>2:
                            yz=float(words[2])
                        xlo = xlo-min([0,xy,xz,xy+xz])
                        xhi = xhi-max([0,xy,xz,xy+xz])
                        ylo = ylo-min([0,yz])
                        yhi = yhi-max([0,yz])
                        a0=xhi-xlo
                        b1=yhi-ylo
                        c2=zhi-zlo
                        b0=xy
                        c0=xz
                        c1=yz

                        cell_count=0
                        in_cell = False

                        a_vector = [a0, 0.0, 0.0]
                        b_vector = [b0, b1,  0.0]
                        c_vector = [c0, c1,  c2 ]
                        unitcell = config.UnitCell(a_vector=a_vector,b_vector=b_vector,c_vector=c_vector)
                        unitcell.set_length_unit(config.ANGSTROM)
                        atom_config = config.AtomConfig(coordinate_system=config.FRACTIONAL, \
                                      length_unit=config.BOHR,unit_cell=unitcell)
                        ret.append(atom_config)
                        continue
                except ValueError:
                    logger.error("failed to parse the unit cell")
                    in_cell = False
                    continue
            if in_coords:
                if currnatm<0:
                    currnatm=1
                if len(words)<5:
                    continue
                if currnatm>natm:
                    logger.error("too many atoms are defined!")
                    continue
                try:
                    e=words[element_index]
                    x=float(words[2])
                    y=float(words[3])
                    z=float(words[4])
                    atom_config.add_atom(element=e,rx=x,ry=y,rz=z)
                    if currnatm==natm:
                        currnatm=-1
                        in_coords=False
                    else:
                        currnatm += 1
            
                except ValueError:
                    logger.error("failed to parse line : "+line)
                    currnatm=-1
                    in_coords=False
                    continue

        dumpfile.close()
        return ret

    def supports_import(self):
        return True

    def supports_export(self):
        return False

    def get_name(self):
        return 'LAMMPS output format'

    def get_defaultname(self,args):
        return "dump"




''' various utility functions and classes '''

import logging
logger = logging.getLogger("piou.util.pyutil")

import os
import sys

try:
    import readline
except ImportError:
    pass
else:
    readline.parse_and_bind("tab: complete")

try:
    input = raw_input
except NameError:
    pass

class Callable:
    ''' use this class to generate so-called class methods.
        here's an example:

        Class Something
            ...
            ...
            def do_something():
                ...
                ...

            from psf.util import pyutil
            do_something = pyutil.Callable(do_something)

        Something.do_something() will then behave like a class method in Java. '''
    def __init__(self, anycallable):
        self.__call__ = anycallable

class Singleton(type):
    ''' python-way of realizing the Singleton pattern. By doing
        class Boo():
            __metaclass__ = pyutil.Singleton
            def ...
        class Boo will behave as a Singleton. '''
    def __init__(self, *args):

        type.__init__(self, *args)

        self._instance = None

    def __call__(self, *args):
        if self._instance is None :
            self._instance = type.__call__(self, *args)
        return self._instance

def my_import(name,ignore_error=False):
    ''' get module under the specified package. '''
    mod = None
    try:
        mod = __import__(name)
    except Exception:
        if not ignore_error:
            logger.error('import error!')
        return None

    components = name.split('.')
    for comp in components[1:]:
        mod = getattr(mod, comp)
    return mod

def map_package_to_dir(package_name):
    '''
    obtain the path to a specific package.
    will return None if no such package exists.
    '''
    boo = my_import(package_name,True)
    if boo != None:
        return boo.__path__[0]
    logger.error('  could not find package : '+package_name)
    return None

def get_all_modules(packageName):
    ''' obtain a list of all modules under the specified package
        parameter : packageName the name of the desired package
        will return None if packageName does not exist. '''
    import glob
    import os,re
    dir = map_package_to_dir(packageName)
    if dir==None:
        return None
    files = os.listdir(dir)
    reg = re.compile(".*\.py$",re.IGNORECASE)
    files = filter(reg.search, files)
    mods = []
    for file in files:
        mods.append(file[:-3])

    ret = []
    for mod in mods:
        if mod!='__init__':
            ret.append(my_import(packageName+'.'+mod,True))
    return ret

def get_all_methods(mod,prefix=''):
    ''' get all methods defined in the specified module.
        parameter : mod the desired module object '''
    ret =[]
    for method in dir(mod):
        attr = getattr(mod,method)
        if callable(attr):
            if type(attr)==type(get_all_methods):
                if method.startswith(prefix):
                    ret.append(attr)
    return ret

def execute(commands,currdir=None,log_offset=''):
    ''' execute commands according to the instructions given by the arg.
        the instruction object is a simple list. here's an example:

        from psf.util import pyutil
        commands=[['environ', PATH boo],['command', bar],['command', hoge]]
        pyutil.execute(commands)

        this example will first set the environment variable PATH to boo,
        then execute the command bar, and will finally execute the command hoge.
        The standard output will be redirected
        to the logger's info method, while the standard error output will be
        redirected to the logger's error method. '''
    import os
    import time
    import subprocess
    for command in commands:
        if command[0].strip()== 'environ':
            line = command[1].split()
            if len(line)==2:
                os.environ[line[0]] = line[1]
                logger.info(log_offset+"environment variable "+line[0]+" was set to "+line[1])

    for command in commands:
        if command[0].strip() == 'command':
            if currdir==None:
                logger.info(log_offset+"executing command : ["+command[1]+']')
            else:
                logger.info(log_offset+'executing command : ['+command[1]+'] under '+str(currdir))
            try:
                popen=subprocess.Popen(command[1],cwd=currdir,stdout=subprocess.PIPE,stderr=subprocess.PIPE,shell=True)
                while 1:
                    line = popen.stderr.readline()
                    if not line: break
                    logger.error(log_offset+line)
                while 1:
                    line=popen.stdout.readline()
                    if not line: break
                    logger.info(log_offset+line)

            except OSError:
                logger.error(log_offset+'failed execution of : '+command[1])

def get_all_classes(mod):
    ''' get all classes defined under the specified module.
        parameter : mod the desired module object '''
    ret=[]
    for clas in dir(mod):
        attr = getattr(mod,clas)
        if callable(attr):
            if type(getattr(mod,clas))==type(OSError):
                ret.append(attr)
    return ret

def get_homedir():
    ''' obtain the home directory of the system.
        will return $HOME for UNIX, %userprofile% for WinNT.
        won't work for Win9x (but who cares!) '''
    import os
    if os.name=='nt':
        return os.environ["userprofile"]
    return os.environ["HOME"]

def parse_bool(str,default=None):
    ''' convert string value to a Boolean. The rools are:
        None or len(str)==0 -> False if 'default' is None, or 'default'
        'true','yes','on','1' -> True
        'false','no','off','0' -> False
        defaults to False. '''
    if isinstance(str,bool):
        return str
    if str==None or len(str)==0:
        if default is None or not isinstance(default,bool):
            return False
        else:
            return default
    if str.lower()== 'yes':
        return True
    if str.lower()=='no':
        return False
    if str.lower()=='on':
        return True
    if str.lower()=='off':
        return False
    if str=='0':
        return False
    if str=='1':
        return True
    if str.lower()=='true':
        return True
    if str.lower()=='false':
        return False
    return False

def is_bool(str):
    ''' determine whether the string given in arg
        is of type boolean.
        parameter : str the string to be tested.'''
    str = str.lower()
    return str=='yes' or str=='no' or str=='0' or str=='1' or str=='on' or str=='off' or str=='true' or str=='false'

def get_ws(length):
    ''' return white-space characters '''
    ret =''
    for i in range(length):
        if i==0:
            continue
        ret += '  '
    return ret
    
def pathsplit(p, rest=[]):
    (h,t) = os.path.split(p)
    if len(h) < 1: return [t]+rest
    if len(t) < 1: return [h]+rest
    return pathsplit(h,[t]+rest)

def commonpath(l1, l2, common=[]):
    if len(l1) < 1: return (common, l1, l2)
    if len(l2) < 1: return (common, l1, l2)
    if l1[0] != l2[0]: return (common, l1, l2)
    return commonpath(l1[1:], l2[1:], common+[l1[0]])

def relpath(p1, p2):
    if p1.endswith(os.path.sep):
        pl = pl[:len(pl)-1]
    if p2.endswith(os.path.sep):
        p2 = p2[:len(p2)-1]
    (common,l1,l2) = commonpath(pathsplit(p1), pathsplit(p2))
    p = []
    if len(l1) > 0:
        p = [ '../' * len(l1) ]
    p = p + l2
    return os.path.join( *p )

def interactive(msg,condition=None,typ=None,default=None,choice=None,return_index=True,skippable=False,\
    msg_invalid_input=None,ask_mode=False,ask_mode_msg=None):
    default_str='['+str(default)+']: '
    if len(str(default))==0:
        default_str = ': '
    while True:
        print
        line = None
        if choice is not None:
            print(msg)
            prestr="("
            for ind in range(len(choice)):
                choi = choice[ind]
                print(str(str(ind)+". ").ljust(4)+choi)
                if ind <len(choice)-1:
                    prestr+=str(ind)+"/"
                else:
                    if not skippable:
                        prestr+=str(ind)+"/x) "+default_str
                    else:
                        prestr+=str(ind)+"/a/x) "+default_str
            if skippable:
                print('a.'.ljust(4)+'apply the default settings here after')
            print('x.'.ljust(4)+'Exit')
            #line = raw_input('Please enter a selection '+prestr)
            line = input('Please enter a selection '+prestr)
            if str(line).strip()=='x':
                sys.exit()
            if skippable and str(line).strip()=='a':
                return None
            if len(str(line).strip())==0:
                if not return_index:
                    return choice[default]
                else:
                    return default
            ichoi=0
            try:
                ichoi=int(str(line).strip())
            except ValueError:
                print('Enter an integer')
                continue
            if ichoi<0 or ichoi>=len(choice):
                print('Invalid choice : '+str(ichoi))
                continue
            if not return_index:
                return choice[ichoi]
            else:
                return ichoi
        else:
            msgstr = "Please enter "+msg+", or type x to exit. "+default_str 
            if skippable:
                msgstr = "Please enter "+msg+\
                ", or type a to apply the default settings here after, or type x to exit. "+default_str 
            #line = raw_input(msgstr)
            line = input(msgstr)
            print(line)
            if str(line).strip()=='x':
                sys.exit()
            elif skippable and str(line).strip()=='a':
                return None
            if (len(line)==0 and default is not None) or condition is not None and condition(line):
                if len(line)==0 and default is not None:
                    line = default
                if not condition(line):
                    if msg_invalid_input is None:
                        print('invalid input : '+line)
                    else:
                        print(msg_invalid_input+line)
                        if ask_mode:
                            ll=''
                            if ask_mode_msg==None:
                                #ll=raw_input('overwrite? (y/n) [n] ')
                                ll=input('overwrite? (y/n) [n] ')
                            else:
                                #ll=raw_input(ask_mode_msg)
                                ll=input(ask_mode_msg)
                            if ll.lower().startswith('y'):
                                return str(line).strip()
                    continue
                if typ==int:
                    try:
                        val=int(str(line).strip())
                    except ValueError:
                        print('enter an int')
                        continue
                elif typ==float:
                    try:
                        val=float(str(line).strip())
                    except ValueError:
                        print('enter a float')
                        continue
                elif typ==bool:
                    val=parse_bool(line)
                    if val is None:
                        print('enter a bool')
                        continue
                else:
                    val=str(line).strip()
                return val
            if msg_invalid_input is None:
                print('invalid input : '+line)
            else:
                print(msg_invalid_input+line)
                if ask_mode:
                    ll=''
                    if ask_mode_msg==None:
                        #ll=raw_input('overwrite? (y/n) [n] ')
                        ll=input('overwrite? (y/n) [n] ')
                    else:
                        #ll=raw_input(ask_mode_msg)
                        ll=input(ask_mode_msg)
                    if ll.lower().startswith('y'):
                        return str(line).strip()
def getTerminalSize():
    def ioctl_GWINSZ(fd):
        try:
            import fcntl, termios, struct, os
            cr = struct.unpack('hh', fcntl.ioctl(fd, termios.TIOCGWINSZ,
        '1234'))
        except:
            return None
        return cr
    cr = ioctl_GWINSZ(0) or ioctl_GWINSZ(1) or ioctl_GWINSZ(2)
    if not cr:
        try:
            fd = os.open(os.ctermid(), os.O_RDONLY)
            cr = ioctl_GWINSZ(fd)
            os.close(fd)
        except:
            pass
    if not cr:
        try:
            cr = (os.environ['LINES'], os.envron['COLUMNS'])
        except:
            cr = (25, 160)
    return int(cr[0]), int(cr[1])

from threading import Thread
class TailF(Thread):
    ''' class which has the same functionality as 'tail -f'.
        !!!UNFINISHED!!! '''
    def __init__(self,filename):
        Thread.__init__(self)
        self.filename = filename
        self.bufsize = 5
        self.lines = []

    def run(self):
        import time, os
        import os.path
        file = open(self.filename,'rb')
        logger.debug("starting tail -f "+self.filename)
        while 1:
            where = file.tell()
            line = file.readline()
            if not line:
                time.sleep(1)
                file.seek(where)
            else:
                print(line,) # already has newline
                #self.lines.append[line]
                #if(len(self.lines)==bufsize):
                #    for l in self.lines:
                #        print l,
                #    self.lines = []



import logging
logger = logging.getLogger('util')

def resolve_int_list(intlist,minVal=None,maxVal=None):
    ''' resolve integer list. for example, if the given string is
       "0-2,4-5,3-6,9-11", then this function will return 
       [0,1,2,3,4,5,6,9,10,11]. will return None if any errors are encountered. 
        arg intlist  : the 'integer list string'  to be resolved. '''
    spllist = intlist.split(",")
    ret = []
    for bb in spllist:
        if len(bb.strip())==0:
            continue
        b = bb.split('-')
        intblock = []
        for ba in b:
            if len(ba.strip())==0:
                continue
            try:
                intblock.append(int(ba))
            except ValueError:
                return None
        if len(intblock)==0:
            continue
        if len(intblock)==1:
            ret.append(intblock[0])
        else:
            intblock = [intblock[0],intblock[1]]
            intblock.sort()
            ndata = intblock[1]-intblock[0]+1
            for i in range(ndata):
                ret.append(i+intblock[0])
        if maxVal is not None:
            for i in ret:
                if i>maxVal:
                    return None
        if minVal is not None:
            for i in ret:
                if i<minVal:
                    return None
    ret = list(set(ret))
    ret.sort()
    return ret




''' manipulate OpenMX I/O files '''
import logging
logger = logging.getLogger('piou.config.openmx')

from piou import config
from piou.config import xyz

class OMX_input(config.AtomConfigGenerator):
    inpfile=None
    def __init__(self,coord_file):
        if coord_file is None:
            self.set_is_valid(False)
            return
        self.inpfile=coord_file

    def _gen_atomic_configuration(self):
        coord_key = 'Atoms.SpeciesAndCoordinates'
        cell_key  = 'Atoms.UnitVectors'
        file = open(self.inpfile,'r')
        inp={}
        in_table=False
        table_name=None
        for line in file:
            line=line.strip()
            if line.startswith('#'):
                continue
            line=line.split('#')[0].strip()
            if not (line.startswith('<') or line.startswith('>')):
                words=line.split()
                if len(words)>1:
                    inp[words[0]] = words[1]
            elif line.startswith('<'):
                table_name=line[1:]
                inp[table_name] = []
                in_table=True
            elif line.endswith('>'):
                in_table=False
            if in_table and not (line.startswith('<') or line.startswith('>')):
                inp[table_name].append(line)
        file.close()

        logger.debug("the OpenMX input file dict")
        logger.debug(str(inp))

        coord_unit = 'Ang'
        if coord_key+'.Unit' in inp:
            coord_unit = inp[coord_key+'.Unit']

        cell_unit='Ang'
        if cell_key+'.Unit' in inp:
            cell_unit = inp[cell_key+'.Unit']

        coord_unit_p = config.ANGSTROM
        coord_sys_p = config.CARTESIAN
        if coord_unit=='AU':
            coord_unit_p = config.BOHR
        if coord_unit=='FRAC':
            coord_sys_p = config.FRACTIONAL
        conf = config.AtomConfig(coordinate_system=coord_sys_p, length_unit=coord_unit_p)

        cell_unit_p = config.ANGSTROM
        if cell_unit=='AU':
            cell_unit_p = config.BOHR
        ucell = config.UnitCell()
        ucell.set_length_unit(cell_unit_p)
        conf.set_unit_cell(ucell)
        if cell_key in inp:
            cellvecs = inp[cell_key]
            if len(cellvecs)<3:
                logger.error('invalid cell : '+str(cellvecs))
            else:
                avec=cellvecs[0].split()
                bvec=cellvecs[1].split()
                cvec=cellvecs[2].split()
                if len(avec)>=3 and len(bvec)>=3 and len(cvec)>=3:
                    try:
                        ucell.set_cell(\
                        a_vector=[float(avec[0]),float(avec[1]),float(avec[2])],\
                        b_vector=[float(bvec[0]),float(bvec[1]),float(bvec[2])],\
                        c_vector=[float(cvec[0]),float(cvec[1]),float(cvec[2])])
                    except ValueError:
                        logger.error('invalid cell vector : '+\
                        str(cellvecs[0])+" "+str(cellvecs[1])+" "+str(cellvecs[2]))

        if coord_key in inp:
            atomlist = inp[coord_key]
            for atom in atomlist:
                ats = atom.split()
                if len(ats)<5:
                    continue
                elem = ats[1]
                try:
                    rx = float(ats[2])
                    ry = float(ats[3])
                    rz = float(ats[4])
                    conf.add_atom(element=elem,rx=rx,ry=ry,rz=rz)
                except ValueError:
                    logger.error('invalid atom specification : '+str(atom))
                
        return [conf]

    def export_atomic_configuration(self,atomic_coordinates,to_file=None,frame_no=None,all_frames=False):
        frameno=config.resolve_frame_no(frame_no,len(atomic_coordinates))
        atomic_coordinate = atomic_coordinates[frameno]
        file=open(to_file,mode='w')
        nat=atomic_coordinate.get_num_atom()
        file.write('Atoms.Number '+str(nat)+'\n')
        file.write('Atoms.SpeciesAndCoordinates.Unit Ang\n')
        file.write('<Atoms.SpeciesAndCoordinates\n')
        atom_list=atomic_coordinate.get_atom_list()
        for i in range(nat):
            atom = atom_list[i]
            elemname = atom.get_element_name()
            [rx,ry,rz] = atom.get_pos(mode=config.CARTESIAN, to_unit=config.ANGSTROM)
            file.write(str(i+1)+' '+elemname+' '+str(rx)+' '+str(ry)+' '+str(rz)+'\n')
        file.write('Atoms.SpeciesAndCoordinates>\n')
        if atomic_coordinate.is_cell_ready():
            file.write('Atoms.UnitVectors.Unit Ang\n')
            (avec,bvec,cvec) = atomic_coordinate.get_unit_cell().get_lattice_vector(to_unit=config.ANGSTROM)
            file.write('<Atoms.UnitVectors\n')
            file.write(str(avec[0])+' '+str(avec[1])+' '+str(avec[2])+'\n')
            file.write(str(bvec[0])+' '+str(bvec[1])+' '+str(bvec[2])+'\n')
            file.write(str(cvec[0])+' '+str(cvec[1])+' '+str(cvec[2])+'\n')
            file.write('Atoms.UnitVectors>\n')
        file.flush()
        file.close()

    def get_name(self):
        return 'OpenMX input format'

    def get_defaultname(self,args):
        return 'omx.dat'

import os 
from piou.config import cif
class OMX_output(config.AtomConfigGenerator):
    mdfile=None
    mdprefix=None

    def supports_export(self):
        return False

    def __init__(self,coord_file):
        if coord_file is None:
            self.set_is_valid(False)
            return
        self.mdfile = coord_file
        self.mdprefix = os.path.basename(coord_file).split('.')[0]

    def supports_frame(self):
        return True

    def _gen_atomic_configuration(self):
        ci = self.mdprefix+'.cif'
        unitcell=None
        if os.path.exists(ci):
            cifcoord=cif.CIF(coord_file=self.mdprefix+".cif")
            ac = cifcoord.get_atomic_configuration()
            unitcell = ac.get_unit_cell()
        fra = []
        f = open(self.mdfile,'r')
        natm=0
        for line in f:
            line = line.strip()
            try:
                natm = int(line)
                conf = config.AtomConfig(coordinate_system=config.CARTESIAN,\
                length_unit=config.ANGSTROM,unit_cell=unitcell)
                fra.append(conf)
                continue
            except ValueError:
                pass
            if line.startswith('time'):
                continue
            words = line.split()
            if len(words)>=4:
                elem = words[0]
                try:
                    rx=float(words[1])
                    ry=float(words[2])
                    rz=float(words[3])
                    conf.add_atom(element=elem,rx=rx,ry=ry,rz=rz)
                except ValueError:
                    logger.error('invalid line  : '+line)
                    continue
        f.close()

        return fra

    def export_atomic_configuration(self,atomic_coordinates,to_file=None,frame_no=None,all_frames=True):
        ''' the xxx.md file seems to be equivalent to the xyz format... '''
        xyzfile=xyz.XYZ(to_file)
        xyzfile.export_atomic_configuration(atomic_coordinates,to_file,frame_no,all_frames)

    def get_name(self):
        return 'OpenMX output format'

    def get_defaultname(self,args):
        return 'omx.md'

